#include <xen/elfnote.h>
#include <xen/elfstructs.h>

#include "list.h"

/* ------------------------------------------------------------- */

#ifndef offsetof
#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
#endif

#define container_of(ptr, type, member) ({                      \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
        (type *)( (char *)__mptr - offsetof(type,member) );})

/* ------------------------------------------------------------- */

#define BUFSIZE 256

struct xendev;

struct devops {
    size_t size;
    int    (*alloc)(struct xendev *xendev);
    int    (*setup)(struct xendev *xendev);
    int    (*xs_be)(struct xendev *xendev, char *node, char *val);
    int    (*xs_fe)(struct xendev *xendev, char *node, char *val);
    int    (*event)(struct xendev *xendev);
    int    (*free)(struct xendev *xendev);
};

struct xendev {
    char               *be_root;
    int                dom;
    int                dev;
    enum xenbus_state  be_state;
    enum xenbus_state  fe_state;
    char               *fe;
    char               *protocol;
    int                remote_port;
    int                local_port;

    struct xs_handle   *xs;
    struct devops      *ops;
    struct list_head   next;
};

/* ------------------------------------------------------------- */

extern int evtchnd;
extern int gnttabdev;

int write_be_str(struct xendev *xendev, char *node, char *val);
int write_be_int(struct xendev *xendev, char *node, int ival);
char *read_be_str(struct xendev *xendev, char *node);
int read_be_int(struct xendev *xendev, char *node);
char *read_fe_str(struct xendev *xendev, char *node);
int read_fe_int(struct xendev *xendev, char *node);

int change_state_xendev(struct xendev *xendev, enum xenbus_state state);

int mainloop(struct devops *ops, char *be_name);
